/*
Copyright 2018 Toyota Research Institute North America


Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors 
may be used to endorse or promote products derived from this software without 
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package fastsimj2;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

public class FSJSimConstants {
	public float airDensity,gravity,mphPerMps,h2KWhPerKg,kWhPerGGE,kWhPerGalDiesel,metersPerMile,inchesPerMeter,gasKWhPerKg,dieselKWhPerKg,litersPerGal;

	//Default constructor
	public FSJSimConstants() {
		airDensity = 1.2f;			//kg/m3
		gravity = 9.81f;			//m/s2
		mphPerMps = 2.2369f; 		//Miles per hour / meters per second
		h2KWhPerKg = 32.72f;		//kWhr per kilogram of hydrogen
		kWhPerGGE = 33.7f;			//kWhr per gallon gasoline equivalent
		kWhPerGalDiesel = 37.9527f;	//kWhr per gallon Diesel
		metersPerMile = 1609;		//Meters per mile
		inchesPerMeter = 39.37f;	//Inches per meter
		gasKWhPerKg = 13.1f;		//kWh per kilogram of Gasoline
		dieselKWhPerKg = 12.61f;	//kWh per kilogram of Diesel
		litersPerGal = 3.7854f;		//Liters per gallon
	}
	//Constructor via CSV file
	public FSJSimConstants(String fname) {
		readFromCSVFile(fname);
	}
	//Function to write contents to CSV file
	public void writeToCSVFile(String fname) {
		try {
			String lsep = System.getProperty("line.separator");
			FileWriter fWriter = new FileWriter(fname);
		
			fWriter.append(headerString());
			fWriter.append(lsep);
			
			fWriter.append(toString());
			fWriter.append(lsep);
			
			fWriter.flush();
			fWriter.close();
		} catch (IOException e) {}
	}
	//Function to read contents from CSV file
	public void readFromCSVFile(String fname) {
		try {
			BufferedReader readingBuffer=new BufferedReader(new FileReader(fname));
			String readLine = readingBuffer.readLine();
			readLine = readingBuffer.readLine();
			parseFromString(readLine);		
			readingBuffer.close();
		} catch (IOException e) {}
	}
	//Header string
	public static String headerString() {
		return "airDensity,gravity,mphPerMps,h2KWhPerKg,kWhPerGGE,kWhPerGalDiesel,metersPerMile,inchesPerMeter,gasKWhPerKg,dieselKWhPerKg,litersPerGal";
	}
	//Form Equivalent string
	@Override public String toString() {
		return ""+airDensity+","+gravity+","+mphPerMps+","+h2KWhPerKg+","+kWhPerGGE+","+kWhPerGalDiesel+","+metersPerMile+","+
				inchesPerMeter+","+gasKWhPerKg+","+dieselKWhPerKg+","+litersPerGal;
	}
	//Function to parse values from equivalent string
	public void parseFromString(String readLine) {
		String[] strSplit = readLine.split(",");
		
		airDensity = Float.parseFloat(strSplit[0]);
		gravity = Float.parseFloat(strSplit[1]);
		mphPerMps = Float.parseFloat(strSplit[2]);
		h2KWhPerKg = Float.parseFloat(strSplit[3]);
		kWhPerGGE = Float.parseFloat(strSplit[4]);
		kWhPerGalDiesel = Float.parseFloat(strSplit[5]);
		metersPerMile = Float.parseFloat(strSplit[6]);
		inchesPerMeter = Float.parseFloat(strSplit[7]);
		gasKWhPerKg = Float.parseFloat(strSplit[8]);
		dieselKWhPerKg = Float.parseFloat(strSplit[9]);
		litersPerGal = Float.parseFloat(strSplit[10]);
	}
}
