/*
Copyright 2018 Toyota Research Institute North America


Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors 
may be used to endorse or promote products derived from this software without 
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package fastsimj2;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

public class FSJVehModelsTable {
	
	//Array of vehicle models
	private FSJVehicleModelParameters[] vehModels;
	
	//Function to return the number of available vehicle models
	public int numVehicleModels() {return vehModels.length;}	
	//Function to return a vehicle model (by its table ID)
	public FSJVehicleModelParameters getVehicleModel(int id) {return vehModels[id];}
	
	//Constructor, reads a Table of FSJVehicleModelParameters
	public FSJVehModelsTable(String fname) {
		readFromFile(fname);
	}
	public FSJVehModelsTable(FSJVehicleModelParameters[] vModels) {
		copyFrom(vModels);
	}
	public FSJVehModelsTable(FSJVehModelsTable other) {
		copyFrom(other);
	}
	
	//Function to set to an Array of vehicle models
	public void copyFrom(FSJVehicleModelParameters[] vModels) {
		vehModels = new FSJVehicleModelParameters[vModels.length];
		for (int i=0; i<vehModels.length; i++) vehModels[i] = new FSJVehicleModelParameters(vModels[i]);
	}
	//Function to copy from another
	public void copyFrom(FSJVehModelsTable otherTable) {
		copyFrom(otherTable.vehModels);
	}
	
	//Function to read from file
	public void readFromFile(String fname) {
		ArrayList<FSJVehicleModelParameters> lst = new ArrayList<FSJVehicleModelParameters>();
		try {
			BufferedReader readingBuffer=new BufferedReader(new FileReader(fname));
			String readLine = readingBuffer.readLine();
			
			while ((readLine = readingBuffer.readLine())!=null) {
				lst.add(new FSJVehicleModelParameters(readLine));
			}
			
			readingBuffer.close();
		} catch (IOException e) {}
		
		vehModels = new FSJVehicleModelParameters[lst.size()];
		for (int i=0; i<vehModels.length; i++) vehModels[i] = lst.get(i);
	}	
	//Function to write to file
	public void writeToFile(String fname) {
		try {
			FileWriter fWriter = new FileWriter(fname);
			String lsep = System.getProperty("line.separator");
			
			fWriter.append(headerString());
			fWriter.append(lsep);
			
			for (int i=0; i<vehModels.length; i++) {
				fWriter.append(vehModels[i].toString());
				fWriter.append(lsep);
			}
			
			fWriter.flush();
			fWriter.close();
		} catch (IOException e) {}
	}
	private static String headerString() {
		return "Scenario name,vehPtType,dragCoef,frontalAreaM2,gliderKg,vehCgM,driveAxleWeightFrac,wheelBaseM,cargoKg,maxFuelStorKw,fuelStorSecsToPeakPwr,fuelStorKwh,fuelStorKwhPerKg,maxFuelConvKw,fcEffType,fuelConvSecsToPeakPwr,fuelConvKwPerKg,maxMotorKw,motorPeakEff,motorSecsToPeakPwr,mcPeKgPerKw,mcPeBaseKg,maxEssKw,useMotorKwInsteadOfESS,maxEssKwh,essKgPerKwh,essBaseKg,essRountTripEff,essLifeCoefA,essLifeCoefB,wheelInteriaKgM2,numWheels,wheelRrCoef,wheelRadiusM,wheelCoefOfFric,controlStrategy,minSoc,maxSoc,minSocMph,maxSocMph,assistKwPerPercOff,chgKwPerPercOff,percUsableSocPassBuf,percFcPwrDoneElec,percHighAccBuf,mphFcOn,kwDemandFcOn,altEff,chgEff,auxKw,transKg,transEff,essToFuelOkError,maxRegen";
	}
}
